﻿using System.IO;
using System.Collections;
using System.Collections.Generic;

namespace SilverNBTLibrary
{
    public class NBTTagCompound : NBTBase
    {
        private Dictionary<string, NBTBase> tagMap = new Dictionary<string, NBTBase>();

        public NBTTagCompound()
            : base("")
        {

        }

        public NBTTagCompound(string name) : base(name) { }

        public override byte GetId()
        {
            return (byte)10;
        }

        public override object GetResult()
        {
            return tagMap;
        }

        public override void WriteData(BinaryWriter bw)
        {
            foreach (NBTBase value in tagMap.Values)
            {
                NBTBase.WriteNamedTag(value, bw);
            }
            bw.WriteHelper((byte)0);
        }

        public override void LoadData(BinaryReader br)
        {
            tagMap.Clear();
            NBTBase tag;
            while ((tag = NBTBase.ReadNamedTag(br)).GetId() != 0)
            {
                tag.Owner = this;
                tagMap.Add(tag.Name, tag);
            }
        }

        public override bool SetValue(object obj)
        {
            if (obj is Dictionary<string, NBTBase>)
            {
                this.tagMap = (Dictionary<string, NBTBase>)obj;
                return true;
            }
            return false;
        }

        public Dictionary<string, NBTBase> GetCollection()
        {
            return tagMap;
        }

        public ICollection GetKeys()
        {
            return tagMap.Keys;
        }

        public ICollection GetValues()
        {
            return tagMap.Values;
        }

        public void SetTag(string name, NBTBase value)
        {
            tagMap.Add(name, value.SetName(name));
            tagMap[name].Owner = this;
        }

        public void SetByte(string name, byte value)
        {
            tagMap.Add(name, new NBTTagByte(name, value));
            tagMap[name].Owner = this;
        }

        public void SetShort(string name, short value)
        {
            tagMap.Add(name, new NBTTagShort(name, value));
            tagMap[name].Owner = this;
        }

        public void SetInt(string name, int value)
        {
            tagMap.Add(name, new NBTTagInt(name, value));
            tagMap[name].Owner = this;
        }

        public void SetLong(string name, long value)
        {
            tagMap.Add(name, new NBTTagLong(name, value));
            tagMap[name].Owner = this;
        }

        public void SetFloat(string name, float value)
        {
            tagMap.Add(name, new NBTTagFloat(name, value));
            tagMap[name].Owner = this;
        }

        public void SetDouble(string name, double value)
        {
            tagMap.Add(name, new NBTTagDouble(name, value));
            tagMap[name].Owner = this;
        }

        public void SetString(string name, string value)
        {
            tagMap.Add(name, new NBTTagString(name, value));
            tagMap[name].Owner = this;
        }

        public void SetByteArray(string name, byte[] value)
        {
            tagMap.Add(name, new NBTTagByteArray(name, value));
            tagMap[name].Owner = this;
        }

        public void SetCompound(string name, NBTTagCompound value)
        {
            tagMap.Add(name, value.SetName(name));
        }

        public void SetIntArray(string name, int[] value)
        {
            tagMap.Add(name, new NBTTagIntArray(name, value));
        }

        public bool HasKey(string name)
        {
            return tagMap.ContainsKey(name);
        }

        public NBTBase GetTag(string name)
        {
            return tagMap[name] as NBTBase; 
        }

        public byte GetByte(string name)
        {
            return HasKey(name) ? (byte)((NBTTagByte)tagMap[name]).GetResult() : (byte)0;
        }

        public short GetShort(string name)
        {
            return HasKey(name) ? (short)((NBTTagShort)tagMap[name]).GetResult() : (short)0;
        }

        public int GetInt(string name)
        {
            return HasKey(name) ? (int)((NBTTagInt)tagMap[name]).GetResult() : (int)0;
        }

        public long GetLong(string name)
        {
            return HasKey(name) ? (long)((NBTTagLong)tagMap[name]).GetResult() : (long)0;
        }

        public double GetDouble(string name)
        {
            return HasKey(name) ? (double)((NBTTagDouble)tagMap[name]).GetResult() : (double)0;
        }

        public float GetFloat(string name)
        {
            return HasKey(name) ? (float)((NBTTagFloat)tagMap[name]).GetResult() : (float)0;
        }

        public string GetString(string name)
        {
            return HasKey(name) ? (string)((NBTTagString)tagMap[name]).GetResult() : (string)"";
        }

        public byte[] GetByteArray(string name)
        {
            return HasKey(name) ? (byte[])((NBTTagByteArray)tagMap[name]).GetResult() : new byte[0];
        }

        public int[] GetIntArray(string name)
        {
            return HasKey(name) ? (int[])((NBTTagIntArray)tagMap[name]).GetResult() : new int[0];
        }

        public NBTTagCompound GetCompound(string name)
        {
            return HasKey(name) ? (NBTTagCompound)((NBTTagCompound)tagMap[name]) : new NBTTagCompound(name);
        }

        public void RemoveTag(string name)
        {
            tagMap.Remove(name);
        }

        public bool HasValue(NBTBase name)
        {
            return tagMap.ContainsValue(name);
        }

        public int Count
        {
            get { return tagMap.Count; }
        }

        public override string ToString()
        {
            string keys = "";
            foreach (string key in tagMap.Keys)
            {
                keys += key + ";";
            }
            return "" + GetTagName() + " " + keys;
        }

        public override NBTBase Clone()
        {
            NBTTagCompound newTag = new NBTTagCompound(this.Name);
            newTag.Expand = this.Expand;
            Dictionary<string, NBTBase> tags = new Dictionary<string, NBTBase>();
            foreach (string key in this.tagMap.Keys)
            {
                NBTBase ta = this.tagMap[key].Clone();
                tags.Add(key, ta.Clone());
            }
            newTag.SetValue(new Dictionary<string, NBTBase>(tags));
            return newTag;
        }
    }
}
